@@ -5,28 +5,30 @@
 #' @description Reads and writes biological sequences (DNA, RNA, protein) in the FASTA format.
 #' 
 #' @usage readFasta(in.file)
-#' writeFasta(fdta, out.file, width = 80)
+#' writeFasta(fdta, out.file, width = 0)
 #' 
-#' @param in.file url/directory/name of FASTA file to read.
+#' @param in.file url/directory/name of (gzipped) FASTA file to read.
 #' @param fdta A \code{\link{tibble}} with sequence data, see \sQuote{Details} below.
-#' @param out.file Name of FASTA file to create.
+#' @param out.file Name of (gzipped) FASTA file to create.
 #' @param width Number of characters per line, or 0 for no linebreaks.
 #' 
 #' @details These functions handle input/output of sequences in the commonly used FASTA format.
-#' For every sequence it is presumed there is one Header-line starting with a \sQuote{>}.
+#' For every sequence it is presumed there is one Header-line starting with a \sQuote{>}. If
+#' filenames (\code{in.file} or \code{out.file}) have the extension \code{.gz} they will automatically be
+#' compressed/uncompressed.
 #' 
 #' The sequences are stored in a \code{\link{tibble}}, opening up all the possibilities in R for
 #' fast and easy manipulations. The content of the file is stored as two columns, \samp{Header}
 #' and \samp{Sequence}. If other columns are added, these will be ignored by \code{\link{writeFasta}}.
 #' 
-#' Setting \code{width = 0} in \code{\link{writeFasta}} results in no linebreaks in the sequences
+#' The default \code{width = 0} in \code{\link{writeFasta}} results in no linebreaks in the sequences
 #' (one sequence per line).
 #' 
-#' @return \code{\link{readFasta}} returns a \code{\link{tibble}} with the contents of the FASTA
+#' @return \code{\link{readFasta}} returns a \code{\link{tibble}} with the contents of the (gzipped) FASTA
 #' file stored in two columns of text. The first, named \samp{Header}, contains
 #' the headerlines and the second, named \samp{Sequence}, contains the sequences.
 #' 
-#' \code{\link{writeFasta}} produces a FASTA file.
+#' \code{\link{writeFasta}} produces a (gzipped) FASTA file.
 #' 
 #' @author Lars Snipen and Kristian Hovde Liland.
 #' 
@@ -44,7 +46,7 @@
 #' # Make use of dplyr to copy parts of the file to another file
 #' readFasta(fa.file) %>% 
 #'   filter(str_detect(Sequence, "TGA$")) %>% 
-#'   writeFasta(out.file = "TGAstop.fasta", width = 0) -> ok
+#'   writeFasta(out.file = "TGAstop.fasta", width = 80) -> ok
 #' }
 #' 
 #' @keywords sequence FASTA
@@ -52,22 +54,26 @@
 #' @importFrom tibble as_tibble
 #' @importFrom stringr str_c
 #' @importFrom dplyr %>% 
+#' @importFrom data.table fread fwrite
 #' 
 #' @export readFasta
 #' @export writeFasta
 #' 
 readFasta <- function(in.file){
-  if(!is.character(in.file) | length(in.file)>1) stop("The argument in.file must be a single text (a filename)")
+  if(!is.character(in.file) | length(in.file) > 1) stop("The argument in.file must be a single text (a filename)")
   if(file.exists(in.file)){
     in.file <- normalizePath(in.file)
-    read_fasta(in.file) %>% 
-      as_tibble() -> fdta
+    tbl <- fread(in.file, header = F, sep = "\t", data.table = F)
+    hh <- grep("^>", tbl[,1])
+    hhh <- c(hh, nrow(tbl) + 1)
+    tibble(Header = tbl[hh,1],
+           Sequence = sapply(1:length(hh), function(i){str_c(tbl[(hhh[i]+1):(hhh[i+1]-1),1], collapse = "")})) -> fdta
     return(fdta)
   } else {
     stop("Cannot find ", in.file, ", please correct path and/or file name")
   }
 }
-writeFasta <- function(fdta, out.file, width = 80){
+writeFasta <- function(fdta, out.file, width = 0){
   if(!("Header" %in% colnames(fdta))){
     stop("This is not a Fasta object, column Header is lacking\n")
   }
@@ -76,15 +82,19 @@ writeFasta <- function(fdta, out.file, width = 80){
   }
   out.file <- file.path(normalizePath(dirname(out.file)),
                         basename(out.file))
-  if(width == 0){
-    str_c(">", fdta$Header) %>% 
-      cbind(fdta$Sequence) %>% 
-      t() %>% 
-      as.vector() %>% 
-      writeLines(con = out.file)
-    status <- TRUE
+  lst <- vector("list", 2 * nrow(fdta))
+  lst[seq(1, 2 * nrow(fdta), 2)] <- as.list(str_c(">", fdta$Header))
+  if(width > 0){
+    lst[seq(2, 2 * nrow(fdta), 2)] <- lapply(fdta$Sequence, chop, width)
   } else {
-    status <- write_fasta(fdta$Header, fdta$Sequence, out.file, width)
+    lst[seq(2, 2 * nrow(fdta), 2)] <- as.list(fdta$Sequence)
   }
-  invisible(status)
+  fwrite(tibble(unlist(lst)), file = out.file, compress = "auto", col.names = F)
+}
+
+chop <- function(sequence, width){
+  n <- str_length(sequence)
+  s1 <- seq(1, n, width)
+  s2 <- pmin(s1 + width - 1, n)
+  return(str_sub(sequence, s1, s2))
 }
